@@ -123,12 +123,10 @@
 #include <string.h>
 #include <limits.h>
 #include <stdlib.h>
-
 #include <glib/gprintf.h>
-#include "rsvg-private.h"
+#include <glib/gi18n-lib.h>
 
-/* Defined in rsvg_internals/src/load.rs */
-typedef struct RsvgLoad RsvgLoad;
+#include "rsvg.h"
 
 /* Defined in rsvg_internals/src/handle.rs */
 typedef struct RsvgHandleRust RsvgHandleRust;
@@ -136,14 +134,6 @@ typedef struct RsvgHandleRust RsvgHandleRust;
 /* Implemented in rsvg_internals/src/xml.rs */
 typedef struct RsvgXmlState RsvgXmlState;
 
-/* Reading state for an RsvgHandle */
-typedef enum {
-    RSVG_HANDLE_STATE_START,
-    RSVG_HANDLE_STATE_LOADING,
-    RSVG_HANDLE_STATE_CLOSED_OK,
-    RSVG_HANDLE_STATE_CLOSED_ERROR
-} RsvgHandleState;
-
 /* Implemented in rsvg_internals/src/xml.rs */
 extern void rsvg_xml_state_error(RsvgXmlState *xml, const char *msg);
 
@@ -158,10 +148,12 @@ extern double rsvg_handle_rust_get_dpi_y (RsvgHandleRust *raw_handle);
 extern void rsvg_handle_rust_set_dpi_x (RsvgHandleRust *raw_handle, double dpi_x);
 extern void rsvg_handle_rust_set_dpi_y (RsvgHandleRust *raw_handle, double dpi_y);
 extern void rsvg_handle_rust_set_base_url (RsvgHandleRust *raw_handle, const char *uri);
+extern void rsvg_handle_rust_set_base_gfile (RsvgHandleRust *raw_handle, GFile *file);
+extern const char *rsvg_handle_rust_get_base_url (RsvgHandleRust *raw_handle);
 extern GFile *rsvg_handle_rust_get_base_gfile (RsvgHandleRust *raw_handle);
 extern guint rsvg_handle_rust_get_flags (RsvgHandleRust *raw_handle);
 extern void rsvg_handle_rust_set_flags (RsvgHandleRust *raw_handle, guint flags);
-extern RsvgHandleState rsvg_handle_rust_get_load_state (RsvgHandleRust *raw_handle);
+extern guint rsvg_handle_rust_set_testing (RsvgHandleRust *raw_handle, gboolean testing);
 extern gboolean rsvg_handle_rust_read_stream_sync (RsvgHandle *handle,
                                                    GInputStream *stream,
                                                    GCancellable *cancellable,
@@ -177,23 +169,20 @@ extern gboolean rsvg_handle_rust_render_cairo_sub (RsvgHandle *handle,
                                                    cairo_t *cr,
                                                    const char *id);
 extern GdkPixbuf *rsvg_handle_rust_get_pixbuf_sub (RsvgHandle *handle, const char *id);
+extern void rsvg_handle_rust_get_dimensions (RsvgHandle *handle,
+                                             RsvgDimensionData *dimension_data);
+extern gboolean rsvg_handle_rust_get_dimensions_sub (RsvgHandle *handle,
+                                                     RsvgDimensionData *dimension_data,
+                                                     const char *id);
+extern gboolean rsvg_handle_rust_get_position_sub (RsvgHandle *handle,
+                                                   RsvgPositionData *dimension_data,
+                                                   const char *id);
+extern void rsvg_handle_rust_set_size_callback (RsvgHandleRust *raw_handle,
+                                                RsvgSizeFunc size_func,
+                                                gpointer user_data,
+                                                GDestroyNotify destroy_notify);
 
 struct RsvgHandlePrivate {
-    RsvgSizeFunc size_func;
-    gpointer user_data;
-    GDestroyNotify user_data_destroy;
-
-    gchar *base_uri; // Keep this here; since rsvg_handle_get_base_uri() returns a const char *
-
-    gboolean in_loop;		/* see get_dimension() */
-
-    gboolean is_testing; /* Are we being run from the test suite? */
-
-#ifdef HAVE_PANGOFT2
-    FcConfig *font_config_for_testing;
-    PangoFontMap *font_map_for_testing;
-#endif
-
     RsvgHandleRust *rust_handle;
 };
 
@@ -220,16 +209,6 @@ static void
 rsvg_handle_init (RsvgHandle * self)
 {
     self->priv = rsvg_handle_get_instance_private (self);
-
-    self->priv->in_loop = FALSE;
-
-    self->priv->is_testing = FALSE;
-
-#ifdef HAVE_PANGOFT2
-    self->priv->font_config_for_testing = NULL;
-    self->priv->font_map_for_testing = NULL;
-#endif
-
     self->priv->rust_handle = rsvg_handle_rust_new();
 }
 
@@ -238,18 +217,6 @@ rsvg_handle_dispose (GObject *instance)
 {
     RsvgHandle *self = (RsvgHandle *) instance;
 
-    if (self->priv->user_data_destroy) {
-        (*self->priv->user_data_destroy) (self->priv->user_data);
-        self->priv->user_data_destroy = NULL;
-    }
-
-    g_clear_pointer (&self->priv->base_uri, g_free);
-
-#ifdef HAVE_PANGOFT2
-    g_clear_pointer (&self->priv->font_config_for_testing, FcConfigDestroy);
-    g_clear_object (&self->priv->font_map_for_testing);
-#endif
-
     g_clear_pointer (&self->priv->rust_handle, rsvg_handle_rust_free);
 
     G_OBJECT_CLASS (rsvg_handle_parent_class)->dispose (instance);
@@ -270,9 +237,15 @@ rsvg_handle_set_property (GObject * instance, guint prop_id, GValue const *value
     case PROP_DPI_Y:
         rsvg_handle_rust_set_dpi_y (self->priv->rust_handle, g_value_get_double (value));
         break;
-    case PROP_BASE_URI:
-        rsvg_handle_set_base_uri (self, g_value_get_string (value));
+    case PROP_BASE_URI: {
+        const char *str = g_value_get_string (value);
+
+        if (str) {
+            rsvg_handle_set_base_uri (self, str);
+        }
+
         break;
+    }
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (instance, prop_id, pspec);
     }
@@ -526,7 +499,8 @@ rsvg_handle_new_from_file (const gchar *file_name, GError **error)
     GFile *file;
     char *scheme;
 
-    rsvg_return_val_if_fail (file_name != NULL, NULL, error);
+    g_return_val_if_fail (file_name != NULL, NULL);
+    g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
     scheme = g_uri_parse_scheme (file_name);
     if (scheme) {
@@ -680,7 +654,7 @@ rsvg_handle_new_from_stream_sync (GInputStream   *input_stream,
 gboolean
 rsvg_handle_write (RsvgHandle *handle, const guchar *buf, gsize count, GError **error)
 {
-    rsvg_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE, error);
+    g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
     g_return_val_if_fail ((buf != NULL && count != 0) || (count == 0), FALSE);
 
@@ -705,8 +679,8 @@ rsvg_handle_write (RsvgHandle *handle, const guchar *buf, gsize count, GError **
 gboolean
 rsvg_handle_close (RsvgHandle *handle, GError **error)
 {
+    g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-    rsvg_return_val_if_fail (handle, FALSE, error);
 
     return rsvg_handle_rust_close(handle, error);
 }
@@ -747,69 +721,6 @@ rsvg_handle_read_stream_sync (RsvgHandle   *handle,
                                               error);
 }
 
-/* http://www.ietf.org/rfc/rfc2396.txt */
-
-static gboolean
-path_is_uri (char const *path)
-{
-    char const *p;
-
-    if (path == NULL)
-        return FALSE;
-
-    if (strlen (path) < 4)
-        return FALSE;
-
-    if ((path[0] < 'a' || path[0] > 'z') &&
-        (path[0] < 'A' || path[0] > 'Z')) {
-        return FALSE;
-    }
-
-    for (p = &path[1];
-	    (*p >= 'a' && *p <= 'z') ||
-        (*p >= 'A' && *p <= 'Z') ||
-        (*p >= '0' && *p <= '9') ||
-         *p == '+' ||
-         *p == '-' ||
-         *p == '.';
-        p++);
-
-    if (strlen (p) < 3)
-        return FALSE;
-
-    return (p[0] == ':' && p[1] == '/' && p[2] == '/');
-}
-
-static gchar *
-get_base_uri_from_filename (const gchar * filename)
-{
-    gchar *current_dir;
-    gchar *absolute_filename;
-    gchar *base_uri;
-
-    if (g_path_is_absolute (filename))
-        return g_filename_to_uri (filename, NULL, NULL);
-
-    current_dir = g_get_current_dir ();
-    absolute_filename = g_build_filename (current_dir, filename, NULL);
-    base_uri = g_filename_to_uri (absolute_filename, NULL, NULL);
-    g_free (absolute_filename);
-    g_free (current_dir);
-
-    return base_uri;
-}
-
-static gboolean
-is_at_start_for_setting_base_file (RsvgHandle *handle)
-{
-    if (rsvg_handle_rust_get_load_state (handle->priv->rust_handle) == RSVG_HANDLE_STATE_START) {
-        return TRUE;
-    } else {
-        g_warning ("Please set the base file or URI before loading any data into RsvgHandle");
-        return FALSE;
-    }
-}
-
 /**
  * rsvg_handle_set_base_uri:
  * @handle: A #RsvgHandle
@@ -823,28 +734,10 @@ is_at_start_for_setting_base_file (RsvgHandle *handle)
 void
 rsvg_handle_set_base_uri (RsvgHandle * handle, const char *base_uri)
 {
-    gchar *uri;
-    GFile *file;
-
     g_return_if_fail (RSVG_IS_HANDLE (handle));
+    g_return_if_fail (base_uri != NULL);
 
-    if (!is_at_start_for_setting_base_file (handle)) {
-        return;
-    }
-
-    if (base_uri == NULL) {
-        return;
-    }
-
-    if (path_is_uri (base_uri))
-        uri = g_strdup (base_uri);
-    else
-        uri = get_base_uri_from_filename (base_uri);
-
-    file = g_file_new_for_uri (uri ? uri : "data:");
-    rsvg_handle_set_base_gfile (handle, file);
-    g_object_unref (file);
-    g_free (uri);
+    rsvg_handle_rust_set_base_url (handle->priv->rust_handle, base_uri);
 }
 
 /**
@@ -862,33 +755,10 @@ void
 rsvg_handle_set_base_gfile (RsvgHandle *handle,
                             GFile      *base_file)
 {
-    RsvgHandlePrivate *priv;
-    char *uri;
-    GFile *real_base_file;
-
     g_return_if_fail (RSVG_IS_HANDLE (handle));
     g_return_if_fail (G_IS_FILE (base_file));
 
-    if (!is_at_start_for_setting_base_file (handle)) {
-        return;
-    }
-
-    priv = handle->priv;
-
-    uri = g_file_get_uri (base_file);
-    rsvg_handle_rust_set_base_url (priv->rust_handle, uri);
-    g_free (uri);
-
-    /* Obtain the sanitized version */
-
-    real_base_file = rsvg_handle_rust_get_base_gfile (priv->rust_handle);
-    g_free (priv->base_uri);
-
-    if (real_base_file) {
-        priv->base_uri = g_file_get_uri (real_base_file);
-    } else {
-        priv->base_uri = NULL;
-    }
+    rsvg_handle_rust_set_base_gfile (handle->priv->rust_handle, base_file);
 }
 
 /**
@@ -904,7 +774,8 @@ const char *
 rsvg_handle_get_base_uri (RsvgHandle * handle)
 {
     g_return_val_if_fail (handle, NULL);
-    return handle->priv->base_uri;
+
+    return rsvg_handle_rust_get_base_url (handle->priv->rust_handle);
 }
 
 /**
@@ -915,7 +786,8 @@ rsvg_handle_get_base_uri (RsvgHandle * handle)
  *
  * Since: 2.9
  *
- * Deprecated: 2.36
+ * Deprecated: 2.36.  Librsvg does not read the metadata/desc/title elements;
+ * this function always returns #NULL.
  */
 const char *
 rsvg_handle_get_metadata (RsvgHandle * handle)
@@ -933,7 +805,8 @@ rsvg_handle_get_metadata (RsvgHandle * handle)
  *
  * Since: 2.4
  *
- * Deprecated: 2.36
+ * Deprecated: 2.36.  Librsvg does not read the metadata/desc/title elements;
+ * this function always returns #NULL.
  */
 const char *
 rsvg_handle_get_title (RsvgHandle * handle)
@@ -951,7 +824,8 @@ rsvg_handle_get_title (RsvgHandle * handle)
  *
  * Since: 2.4
  *
- * Deprecated: 2.36
+ * Deprecated: 2.36.  Librsvg does not read the metadata/desc/title elements;
+ * this function always returns #NULL.
  */
 const char *
 rsvg_handle_get_desc (RsvgHandle * handle)
@@ -967,32 +841,6 @@ rsvg_handle_get_rust (RsvgHandle *handle)
     return handle->priv->rust_handle;
 }
 
-static gboolean
-is_loaded (RsvgHandle *handle)
-{
-    switch (rsvg_handle_rust_get_load_state (handle->priv->rust_handle)) {
-    case RSVG_HANDLE_STATE_START:
-        g_warning ("RsvgHandle has not been loaded");
-        return FALSE;
-
-    case RSVG_HANDLE_STATE_LOADING:
-        g_warning ("RsvgHandle is still loading; call rsvg_handle_close() first");
-        return FALSE;
-
-    case RSVG_HANDLE_STATE_CLOSED_OK:
-        return TRUE;
-
-    case RSVG_HANDLE_STATE_CLOSED_ERROR:
-        g_warning ("RsvgHandle could not read or parse the SVG; did you check for errors\n"
-                   "during the loading stage?");
-        return FALSE;
-
-    default:
-        g_assert_not_reached();
-        return FALSE;
-    }
-}
-
 /**
  * rsvg_handle_render_cairo_sub:
  * @handle: A #RsvgHandle
@@ -1015,10 +863,6 @@ rsvg_handle_render_cairo_sub (RsvgHandle * handle, cairo_t * cr, const char *id)
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (cr != NULL, FALSE);
 
-    if (!is_loaded (handle)) {
-        return FALSE;
-    }
-
     return rsvg_handle_rust_render_cairo_sub (handle, cr, id);
 }
 
@@ -1056,22 +900,7 @@ rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_d
     g_return_if_fail (RSVG_IS_HANDLE (handle));
     g_return_if_fail (dimension_data != NULL);
 
-    if (!is_loaded (handle)) {
-        return;
-    }
-
-    /* This function is probably called from the cairo_render functions.
-     * To prevent an infinite loop we are saving the state.
-     */
-    if (!handle->priv->in_loop) {
-        handle->priv->in_loop = TRUE;
-        rsvg_handle_get_dimensions_sub (handle, dimension_data, NULL);
-        handle->priv->in_loop = FALSE;
-    } else {
-        /* Called within the size function, so return a standard size */
-        dimension_data->em = dimension_data->width = 1;
-        dimension_data->ex = dimension_data->height = 1;
-    }
+    rsvg_handle_rust_get_dimensions (handle, dimension_data);
 }
 
 /**
@@ -1091,31 +920,10 @@ rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_d
 gboolean
 rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimension_data, const char *id)
 {
-    RsvgRectangle ink_r;
-
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (dimension_data, FALSE);
 
-    if (!is_loaded (handle)) {
-        return FALSE;
-    }
-
-    memset (&ink_r, 0, sizeof (RsvgRectangle));
-    memset (dimension_data, 0, sizeof (RsvgDimensionData));
-
-    if (!rsvg_handle_get_geometry_sub (handle, &ink_r, NULL, id)) {
-        return FALSE;
-    }
-
-    dimension_data->width = ink_r.width;
-    dimension_data->height = ink_r.height;
-    dimension_data->em = dimension_data->width;
-    dimension_data->ex = dimension_data->height;
-
-    if (handle->priv->size_func)
-        (*handle->priv->size_func) (&dimension_data->width, &dimension_data->height,
-                                    handle->priv->user_data);
-    return TRUE;
+    return rsvg_handle_rust_get_dimensions_sub (handle, dimension_data, id);
 }
 
 /**
@@ -1138,10 +946,6 @@ rsvg_handle_get_geometry_sub (RsvgHandle * handle, RsvgRectangle * ink_rect, Rsv
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
 
-    if (!is_loaded (handle)) {
-        return FALSE;
-    }
-
     return rsvg_handle_rust_get_geometry_sub(handle, ink_rect, logical_rect, id);
 }
 
@@ -1162,35 +966,10 @@ rsvg_handle_get_geometry_sub (RsvgHandle * handle, RsvgRectangle * ink_rect, Rsv
 gboolean
 rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_data, const char *id)
 {
-    RsvgRectangle ink_r;
-    int width, height;
-
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
     g_return_val_if_fail (position_data != NULL, FALSE);
 
-    if (!is_loaded (handle)) {
-        return FALSE;
-    }
-
-    memset (position_data, 0, sizeof (*position_data));
-
-    /* Short-cut when no id is given. */
-    if (NULL == id || '\0' == *id)
-        return TRUE;
-
-    if (!rsvg_handle_get_geometry_sub (handle, &ink_r, NULL, id))
-        return FALSE;
-
-    position_data->x = ink_r.x;
-    position_data->y = ink_r.y;
-
-    width = ink_r.width;
-    height = ink_r.height;
-
-    if (handle->priv->size_func)
-        (*handle->priv->size_func) (&width, &height, handle->priv->user_data);
-
-    return TRUE;
+    return rsvg_handle_rust_get_position_sub (handle, position_data, id);
 }
 
 /**
@@ -1210,10 +989,6 @@ rsvg_handle_has_sub (RsvgHandle *handle,
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
 
-    if (!is_loaded (handle)) {
-        return FALSE;
-    }
-
     return rsvg_handle_rust_has_sub (handle, id);
 }
 
@@ -1244,10 +1019,6 @@ rsvg_handle_get_pixbuf_sub (RsvgHandle * handle, const char *id)
 {
     g_return_val_if_fail (RSVG_IS_HANDLE (handle), NULL);
 
-    if (!is_loaded (handle)) {
-        return NULL;
-    }
-
     return rsvg_handle_rust_get_pixbuf_sub (handle, id);
 }
 
@@ -1350,70 +1121,19 @@ rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
 void
 rsvg_handle_set_size_callback (RsvgHandle * handle,
                                RsvgSizeFunc size_func,
-                               gpointer user_data, GDestroyNotify user_data_destroy)
+                               gpointer user_data,
+                               GDestroyNotify user_data_destroy)
 {
     g_return_if_fail (RSVG_IS_HANDLE (handle));
 
-    if (handle->priv->user_data_destroy)
-        (*handle->priv->user_data_destroy) (handle->priv->user_data);
-
-    handle->priv->size_func = size_func;
-    handle->priv->user_data = user_data;
-    handle->priv->user_data_destroy = user_data_destroy;
-}
-
-#ifdef HAVE_PANGOFT2
-
-static void
-create_font_config_for_testing (RsvgHandle *handle)
-{
-    const char *font_paths[] = {
-        "resources/Roboto-Regular.ttf",
-        "resources/Roboto-Italic.ttf",
-        "resources/Roboto-Bold.ttf",
-        "resources/Roboto-BoldItalic.ttf",
-    };
-
-    int i;
-
-    if (handle->priv->font_config_for_testing != NULL)
-        return;
-
-    handle->priv->font_config_for_testing = FcConfigCreate ();
-
-    for (i = 0; i < G_N_ELEMENTS(font_paths); i++) {
-        char *font_path = g_test_build_filename (G_TEST_DIST, font_paths[i], NULL);
-
-        if (!FcConfigAppFontAddFile (handle->priv->font_config_for_testing, (const FcChar8 *) font_path)) {
-            g_error ("Could not load font file \"%s\" for tests; aborting", font_path);
-        }
-
-        g_free (font_path);
-    }
-}
-
-#endif
-
-static void
-rsvg_handle_update_font_map_for_testing (RsvgHandle *handle)
-{
-#ifdef HAVE_PANGOFT2
-    if (handle->priv->is_testing) {
-        create_font_config_for_testing (handle);
-
-        if (handle->priv->font_map_for_testing == NULL) {
-            handle->priv->font_map_for_testing = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT);
-            pango_fc_font_map_set_config (PANGO_FC_FONT_MAP (handle->priv->font_map_for_testing),
-                                          handle->priv->font_config_for_testing);
-
-            pango_cairo_font_map_set_default (PANGO_CAIRO_FONT_MAP (handle->priv->font_map_for_testing));
-        }
-    }
-#endif
+    rsvg_handle_rust_set_size_callback (handle->priv->rust_handle,
+                                        size_func,
+                                        user_data,
+                                        user_data_destroy);
 }
 
 /**
- * _rsvg_handle_internal_set_testing:
+ * rsvg_handle_internal_set_testing:
  * @handle: a #RsvgHandle
  * @testing: Whether to enable testing mode
  *
@@ -1425,18 +1145,7 @@ rsvg_handle_internal_set_testing (RsvgHandle *handle, gboolean testing)
 {
     g_return_if_fail (RSVG_IS_HANDLE (handle));
 
-    handle->priv->is_testing = testing ? TRUE : FALSE;
-
-    rsvg_handle_update_font_map_for_testing (handle);
-}
-
-G_GNUC_INTERNAL
-gboolean rsvg_handle_get_is_testing (RsvgHandle *handle);
-
-gboolean
-rsvg_handle_get_is_testing (RsvgHandle *handle)
-{
-    return handle->priv->is_testing;
+    rsvg_handle_rust_set_testing (handle->priv->rust_handle, testing);
 }
 
 /* This one is defined in the C code, because the prototype has varargs
