@@ -6,9 +6,8 @@ use cssparser::{CowRcStr, Parser, Token};
 use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use error::{NodeError, ValueErrorKind};
-use handle::RsvgHandle;
 use node::{NodeResult, NodeTrait, RsvgNode};
-use parsers::{self, parse, Parse};
+use parsers::{self, Parse, ParseValue};
 use property_bag::PropertyBag;
 use rect::IRect;
 use surface_utils::{
@@ -62,20 +61,15 @@ impl Composite {
 }
 
 impl NodeTrait for Composite {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        handle: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
-        self.base.set_atts(node, handle, pbag)?;
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+        self.base.set_atts(node, pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr {
                 Attribute::In2 => {
                     self.in2.replace(Some(Input::parse(Attribute::In2, value)?));
                 }
-                Attribute::Operator => self.operator.set(parse("operator", value, ())?),
+                Attribute::Operator => self.operator.set(attr.parse(value, ())?),
                 Attribute::K1 => self.k1.set(
                     parsers::number(value).map_err(|err| NodeError::attribute_error(attr, err))?,
                 ),
