@@ -9,10 +9,9 @@ use attributes::Attribute;
 use coord_units::CoordUnits;
 use drawing_ctx::DrawingCtx;
 use error::{RenderingError, ValueErrorKind};
-use handle::RsvgHandle;
 use length::{Length, LengthDir, LengthUnit};
 use node::{NodeResult, NodeTrait, NodeType, RsvgNode};
-use parsers::{parse_and_validate, ParseError};
+use parsers::{ParseError, ParseValue};
 use property_bag::PropertyBag;
 use state::{ColorInterpolationFilters, ComputedValues};
 use surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
@@ -111,12 +110,7 @@ impl Primitive {
 }
 
 impl NodeTrait for Primitive {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        _: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         // With ObjectBoundingBox, only fractions and percents are allowed.
         let primitiveunits = node
             .get_parent()
@@ -147,26 +141,22 @@ impl NodeTrait for Primitive {
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(Some(parse_and_validate(
-                    "x",
+                Attribute::X => self.x.set(Some(attr.parse_and_validate(
                     value,
                     LengthDir::Horizontal,
                     check_units,
                 )?)),
-                Attribute::Y => self.y.set(Some(parse_and_validate(
-                    "y",
+                Attribute::Y => self.y.set(Some(attr.parse_and_validate(
                     value,
                     LengthDir::Vertical,
                     check_units,
                 )?)),
-                Attribute::Width => self.width.set(Some(parse_and_validate(
-                    "width",
+                Attribute::Width => self.width.set(Some(attr.parse_and_validate(
                     value,
                     LengthDir::Horizontal,
                     check_units_and_ensure_nonnegative,
                 )?)),
-                Attribute::Height => self.height.set(Some(parse_and_validate(
-                    "height",
+                Attribute::Height => self.height.set(Some(attr.parse_and_validate(
                     value,
                     LengthDir::Vertical,
                     check_units_and_ensure_nonnegative,
@@ -202,13 +192,8 @@ impl PrimitiveWithInput {
 }
 
 impl NodeTrait for PrimitiveWithInput {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        handle: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
-        self.base.set_atts(node, handle, pbag)?;
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+        self.base.set_atts(node, pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr {
