@@ -1,4 +1,5 @@
 use std::cell::{Cell, Ref, RefCell};
+use std::ffi::CString;
 use std::mem;
 use std::ptr;
 use std::rc::Rc;
@@ -8,7 +9,7 @@ use cairo::{self, ImageSurface, Status};
 use cairo_sys;
 use gdk_pixbuf::{Colorspace, Pixbuf, PixbufLoader, PixbufLoaderExt};
 use gdk_pixbuf_sys;
-use gio::{File as GFile, InputStream};
+use gio::{File as GFile, FileExt};
 use gio_sys;
 use glib::translate::*;
 use glib_sys;
@@ -17,7 +18,7 @@ use url::Url;
 
 use allowed_url::AllowedUrl;
 use css::{self, CssStyles};
-use defs::{Fragment, Href};
+use defs::Href;
 use dpi::Dpi;
 use drawing_ctx::{DrawingCtx, RsvgRectangle};
 use error::{set_gerror, DefsLookupErrorKind, LoadingError, RenderingError};
@@ -33,8 +34,6 @@ use surface_utils::{
 };
 use svg::Svg;
 use util::rsvg_g_warning;
-use xml::XmlState;
-use xml2_load::xml_state_load_from_possibly_compressed_stream;
 
 // A *const RsvgHandle is just an opaque pointer we get from C
 #[repr(C)]
@@ -51,12 +50,19 @@ pub struct RsvgDimensionData {
     ex: f64,
 }
 
+// Keep in sync with rsvg.h:RsvgPositionData
+#[repr(C)]
+pub struct RsvgPositionData {
+    x: libc::c_int,
+    y: libc::c_int,
+}
+
 /// Flags used during loading
 ///
 /// We communicate these to/from the C code with a guint <-> u32,
 /// and this struct provides to_flags() and from_flags() methods.
 #[derive(Default, Copy, Clone)]
-pub struct LoadOptions {
+pub struct LoadFlags {
     /// Whether to turn off size limits in libxml2
     pub unlimited_size: bool,
 
@@ -64,7 +70,18 @@ pub struct LoadOptions {
     pub keep_image_data: bool,
 }
 
-#[repr(C)]
+#[derive(Clone)]
+pub struct LoadOptions {
+    pub flags: LoadFlags,
+    pub base_url: Option<Url>,
+}
+
+impl LoadOptions {
+    fn new(flags: LoadFlags, base_url: Option<Url>) -> LoadOptions {
+        LoadOptions { flags, base_url }
+    }
+}
+
 #[derive(Copy, Clone, PartialEq)]
 pub enum LoadState {
     Start,
@@ -73,13 +90,65 @@ pub enum LoadState {
     ClosedError,
 }
 
+// Keep in sync with rsvg.h:RsvgSizeFunc
+type RsvgSizeFunc = Option<
+    unsafe extern "C" fn(
+        inout_width: *mut libc::c_int,
+        inout_height: *mut libc::c_int,
+        user_data: glib_sys::gpointer,
+    ),
+>;
+
+struct SizeCallback {
+    size_func: RsvgSizeFunc,
+    user_data: glib_sys::gpointer,
+    destroy_notify: glib_sys::GDestroyNotify,
+}
+
+impl SizeCallback {
+    fn new() -> SizeCallback {
+        SizeCallback {
+            size_func: None,
+            user_data: ptr::null_mut(),
+            destroy_notify: None,
+        }
+    }
+
+    fn call(&self, width: libc::c_int, height: libc::c_int) -> (libc::c_int, libc::c_int) {
+        unsafe {
+            let mut w = width;
+            let mut h = height;
+
+            if let Some(ref f) = self.size_func {
+                f(&mut w, &mut h, self.user_data);
+            };
+
+            (w, h)
+        }
+    }
+}
+
+impl Drop for SizeCallback {
+    fn drop(&mut self) {
+        unsafe {
+            if let Some(ref f) = self.destroy_notify {
+                f(self.user_data);
+            };
+        }
+    }
+}
+
 pub struct Handle {
     dpi: Dpi,
     base_url: RefCell<Option<Url>>,
-    svg: RefCell<Option<Svg>>,
-    load_options: Cell<LoadOptions>,
+    base_url_cstring: RefCell<Option<CString>>, // needed because the C api returns *const char
+    svg: RefCell<Option<Rc<Svg>>>,
+    load_flags: Cell<LoadFlags>,
     load_state: Cell<LoadState>,
     load: RefCell<Option<LoadContext>>,
+    size_callback: RefCell<SizeCallback>,
+    in_loop: Cell<bool>,
+    is_testing: Cell<bool>,
 }
 
 impl Handle {
@@ -87,10 +156,49 @@ impl Handle {
         Handle {
             dpi: Dpi::default(),
             base_url: RefCell::new(None),
+            base_url_cstring: RefCell::new(None),
             svg: RefCell::new(None),
-            load_options: Cell::new(LoadOptions::default()),
+            load_flags: Cell::new(LoadFlags::default()),
             load_state: Cell::new(LoadState::Start),
             load: RefCell::new(None),
+            size_callback: RefCell::new(SizeCallback::new()),
+            in_loop: Cell::new(false),
+            is_testing: Cell::new(false),
+        }
+    }
+
+    fn set_base_url(&self, url: &str) {
+        if self.load_state.get() != LoadState::Start {
+            rsvg_g_warning(
+                "Please set the base file or URI before loading any data into RsvgHandle",
+            );
+            return;
+        }
+
+        match Url::parse(&url) {
+            Ok(u) => {
+                let url_cstring = CString::new(u.as_str()).unwrap();
+
+                rsvg_log!("setting base_uri to \"{}\"", u.as_str());
+                *self.base_url.borrow_mut() = Some(u);
+                *self.base_url_cstring.borrow_mut() = Some(url_cstring);
+            }
+
+            Err(e) => {
+                rsvg_log!(
+                    "not setting base_uri to \"{}\" since it is invalid: {}",
+                    url,
+                    e
+                );
+            }
+        }
+    }
+
+    fn set_base_gfile(&self, file: &GFile) {
+        if let Some(uri) = file.get_uri() {
+            self.set_base_url(&uri);
+        } else {
+            rsvg_g_warning("file has no URI; will not set the base URI");
         }
     }
 
@@ -102,38 +210,19 @@ impl Handle {
     ) -> Result<(), LoadingError> {
         self.load_state.set(LoadState::Loading);
 
-        self.read_stream_internal(handle, stream, cancellable)
-            .and_then(|_| {
-                self.load_state.set(LoadState::ClosedOk);
-                Ok(())
-            })
+        let svg = Svg::load_from_stream(&self.load_options(), handle, stream, cancellable)
             .map_err(|e| {
                 self.load_state.set(LoadState::ClosedError);
                 e
-            })
+            })?;
+
+        *self.svg.borrow_mut() = Some(Rc::new(svg));
+        self.load_state.set(LoadState::ClosedOk);
+        Ok(())
     }
 
-    fn read_stream_internal(
-        &mut self,
-        handle: *mut RsvgHandle,
-        stream: gio::InputStream,
-        cancellable: Option<gio::Cancellable>,
-    ) -> Result<(), LoadingError> {
-        let load_options = self.load_options.get();
-
-        let mut xml = XmlState::new(handle);
-
-        xml_state_load_from_possibly_compressed_stream(
-            &mut xml,
-            &load_options,
-            stream,
-            cancellable,
-        )?;
-
-        xml.validate_tree()?;
-
-        *self.svg.borrow_mut() = Some(xml.steal_result());
-        Ok(())
+    fn load_options(&self) -> LoadOptions {
+        LoadOptions::new(self.load_flags.get(), self.base_url.borrow().clone())
     }
 
     pub fn write(&mut self, handle: *mut RsvgHandle, buf: &[u8]) {
@@ -145,7 +234,7 @@ impl Handle {
         if self.load_state.get() == LoadState::Start {
             self.load_state.set(LoadState::Loading);
 
-            self.load = RefCell::new(Some(LoadContext::new(handle, self.load_options.get())));
+            self.load = RefCell::new(Some(LoadContext::new(handle, self.load_options())));
         }
 
         assert!(self.load_state.get() == LoadState::Loading);
@@ -195,7 +284,7 @@ impl Handle {
 
         xml.validate_tree()?;
 
-        *self.svg.borrow_mut() = Some(xml.steal_result());
+        *self.svg.borrow_mut() = Some(Rc::new(xml.steal_result()));
         Ok(())
     }
 
@@ -208,21 +297,19 @@ impl Handle {
 
     fn create_drawing_ctx_for_node(
         &mut self,
-        handle: *mut RsvgHandle,
         cr: &cairo::Context,
         dimensions: &RsvgDimensionData,
         node: Option<&RsvgNode>,
-        is_testing: bool,
     ) -> DrawingCtx {
         let mut draw_ctx = DrawingCtx::new(
-            handle,
+            self.svg.borrow().as_ref().unwrap().clone(),
             cr,
             f64::from(dimensions.width),
             f64::from(dimensions.height),
             dimensions.em,
             dimensions.ex,
-            get_dpi(handle).clone(),
-            is_testing,
+            self.dpi.clone(),
+            self.is_testing.get(),
         );
 
         if let Some(node) = node {
@@ -257,22 +344,11 @@ impl Handle {
         node: &RsvgNode,
     ) -> Result<(RsvgRectangle, RsvgRectangle), RenderingError> {
         let dimensions = self.get_dimensions(handle)?;
-
         let target = ImageSurface::create(cairo::Format::Rgb24, 1, 1)?;
-
         let cr = cairo::Context::new(&target);
-
-        let mut draw_ctx = self.create_drawing_ctx_for_node(
-            handle,
-            &cr,
-            &dimensions,
-            Some(node),
-            is_testing(handle),
-        );
-
+        let mut draw_ctx = self.create_drawing_ctx_for_node(&cr, &dimensions, Some(node));
         let svg_ref = self.svg.borrow();
         let svg = svg_ref.as_ref().unwrap();
-
         let root = svg.tree.root();
 
         draw_ctx.draw_node_from_stack(&root.get_cascaded_values(), &root, false)?;
@@ -304,9 +380,7 @@ impl Handle {
         };
 
         let (node, is_root) = if let Some(id) = id {
-            let n = self
-                .defs_lookup(handle, id)
-                .map_err(RenderingError::InvalidId)?;
+            let n = self.lookup_node(id).map_err(RenderingError::InvalidId)?;
             let is_root = Rc::ptr_eq(&n, &root);
             (n, is_root)
         } else {
@@ -333,17 +407,11 @@ impl Handle {
         self.get_node_geometry(handle, &node)
     }
 
-    fn defs_lookup(
-        &mut self,
-        handle: *const RsvgHandle,
-        name: &str,
-    ) -> Result<RsvgNode, DefsLookupErrorKind> {
+    fn lookup_node(&mut self, id: &str) -> Result<RsvgNode, DefsLookupErrorKind> {
         let svg_ref = self.svg.borrow();
         let svg = svg_ref.as_ref().unwrap();
 
-        let mut defs = svg.defs.borrow_mut();
-
-        let href = Href::with_fragment(name).map_err(DefsLookupErrorKind::HrefError)?;
+        let href = Href::with_fragment(id).map_err(DefsLookupErrorKind::HrefError)?;
 
         match href {
             Href::WithFragment(fragment) => {
@@ -369,7 +437,7 @@ impl Handle {
                     return Err(DefsLookupErrorKind::CannotLookupExternalReferences);
                 }
 
-                match defs.lookup(handle, &fragment) {
+                match svg.lookup_node_by_id(fragment.fragment()) {
                     Some(n) => Ok(n),
                     None => Err(DefsLookupErrorKind::NotFound),
                 }
@@ -379,9 +447,9 @@ impl Handle {
         }
     }
 
-    fn has_sub(&mut self, handle: *const RsvgHandle, name: &str) -> bool {
+    fn has_sub(&mut self, name: &str) -> bool {
         // FIXME: return a proper error; only NotFound should map to false
-        self.defs_lookup(handle, name).is_ok()
+        self.lookup_node(name).is_ok()
     }
 
     fn render_cairo_sub(
@@ -402,10 +470,7 @@ impl Handle {
         }
 
         let node = if let Some(id) = id {
-            Some(
-                self.defs_lookup(handle, id)
-                    .map_err(RenderingError::InvalidId)?,
-            )
+            Some(self.lookup_node(id).map_err(RenderingError::InvalidId)?)
         } else {
             None
         };
@@ -414,17 +479,10 @@ impl Handle {
 
         cr.save();
 
-        let mut draw_ctx = self.create_drawing_ctx_for_node(
-            handle,
-            cr,
-            &dimensions,
-            node.as_ref(),
-            is_testing(handle),
-        );
+        let mut draw_ctx = self.create_drawing_ctx_for_node(cr, &dimensions, node.as_ref());
 
         let svg_ref = self.svg.borrow();
         let svg = svg_ref.as_ref().unwrap();
-
         let root = svg.tree.root();
 
         let res = draw_ctx.draw_node_from_stack(&root.get_cascaded_values(), &root, false);
@@ -487,12 +545,13 @@ const RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA: u32 = 1 << 1;
 
 pub fn get_load_options(handle: *const RsvgHandle) -> LoadOptions {
     let rhandle = get_rust_handle(handle);
-    rhandle.load_options.get()
+
+    rhandle.load_options()
 }
 
-impl LoadOptions {
+impl LoadFlags {
     pub fn from_flags(flags: u32) -> Self {
-        LoadOptions {
+        LoadFlags {
             unlimited_size: (flags & RSVG_HANDLE_FLAG_UNLIMITED) != 0,
             keep_image_data: (flags & RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA) != 0,
         }
@@ -524,47 +583,26 @@ extern "C" {
 
     fn rsvg_handle_get_rust(handle: *const RsvgHandle) -> *mut Handle;
 
-    fn rsvg_handle_get_is_testing(handle: *const RsvgHandle) -> glib_sys::gboolean;
-
     fn rsvg_handle_get_dimensions(handle: *mut RsvgHandle, dimensions: *mut RsvgDimensionData);
 }
 
-fn is_testing(handle: *const RsvgHandle) -> bool {
-    unsafe { from_glib(rsvg_handle_get_is_testing(handle)) }
-}
-
-pub fn lookup_node(handle: *const RsvgHandle, fragment: &Fragment) -> Option<Rc<Node>> {
-    let rhandle = get_rust_handle(handle);
-
-    let svg_ref = rhandle.svg.borrow();
-    let svg = svg_ref.as_ref().unwrap();
-    let mut defs_ref = svg.defs.borrow_mut();
-
-    defs_ref.lookup(handle, fragment)
-}
-
 // Looks up a node by its id.
-//
-// Note that this ignores the Fragment's url; it only uses the fragment identifier.
-pub fn lookup_fragment_id(handle: *const RsvgHandle, fragment: &Fragment) -> Option<Rc<Node>> {
+pub fn lookup_fragment_id(handle: *const RsvgHandle, id: &str) -> Option<Rc<Node>> {
     let rhandle = get_rust_handle(handle);
 
     let svg_ref = rhandle.svg.borrow();
     let svg = svg_ref.as_ref().unwrap();
-    let defs_ref = svg.defs.borrow();
 
-    defs_ref.lookup_fragment_id(fragment.fragment())
+    svg.lookup_node_by_id(id)
 }
 
-pub fn load_extern(handle: *const RsvgHandle, aurl: &AllowedUrl) -> Result<*const RsvgHandle, ()> {
+pub fn load_extern(load_options: &LoadOptions, aurl: &AllowedUrl) -> Result<*const RsvgHandle, ()> {
     unsafe {
-        let rhandle = get_rust_handle(handle);
-
         let file = GFile::new_for_uri(aurl.url().as_str());
 
         let res = rsvg_handle_new_from_gfile_sync(
             file.to_glib_none().0,
-            rhandle.load_options.get().to_flags(),
+            load_options.flags.to_flags(),
             ptr::null(),
             ptr::null_mut(),
         );
@@ -572,24 +610,11 @@ pub fn load_extern(handle: *const RsvgHandle, aurl: &AllowedUrl) -> Result<*cons
         if res.is_null() {
             Err(())
         } else {
-            let rhandle = get_rust_handle(handle);
-
-            let svg_ref = rhandle.svg.borrow();
-            let svg = svg_ref.as_ref().unwrap();
-
-            svg.tree.cascade();
-
             Ok(res)
         }
     }
 }
 
-pub fn get_dpi<'a>(handle: *const RsvgHandle) -> &'a Dpi {
-    let rhandle = get_rust_handle(handle);
-
-    &rhandle.dpi
-}
-
 pub fn get_base_url<'a>(handle: *const RsvgHandle) -> Ref<'a, Option<Url>> {
     let rhandle = get_rust_handle(handle);
 
@@ -601,27 +626,14 @@ pub struct BinaryData {
     pub content_type: Option<String>,
 }
 
-pub fn acquire_data(
-    _handle: *mut RsvgHandle,
-    aurl: &AllowedUrl,
-) -> Result<BinaryData, LoadingError> {
-    io::acquire_data(aurl, None)
-}
-
-pub fn acquire_stream(
-    _handle: *mut RsvgHandle,
-    aurl: &AllowedUrl,
-) -> Result<InputStream, LoadingError> {
-    io::acquire_stream(&aurl, None)
-}
-
 pub fn load_image_to_surface(
-    handle: *mut RsvgHandle,
-    aurl: &AllowedUrl,
+    load_options: &LoadOptions,
+    url: &str,
 ) -> Result<ImageSurface, LoadingError> {
-    let rhandle = get_rust_handle(handle);
+    let aurl = AllowedUrl::from_href(url, load_options.base_url.as_ref())
+        .map_err(|_| LoadingError::BadUrl)?;
 
-    let data = acquire_data(handle, aurl)?;
+    let data = io::acquire_data(&aurl, None)?;
 
     if data.data.len() == 0 {
         return Err(LoadingError::EmptyData);
@@ -640,7 +652,7 @@ pub fn load_image_to_surface(
 
     let surface = SharedImageSurface::from_pixbuf(&pixbuf)?.into_image_surface()?;
 
-    if rhandle.load_options.get().keep_image_data {
+    if load_options.flags.keep_image_data {
         if let Some(mime_type) = data.content_type {
             extern "C" {
                 fn cairo_surface_set_mime_data(
@@ -677,44 +689,34 @@ pub fn load_image_to_surface(
 
 // This function just slurps CSS data from a possibly-relative href
 // and parses it.  We'll move it to a better place in the end.
-pub fn load_css(css_styles: &mut CssStyles, handle: *mut RsvgHandle, href_str: &str) {
-    let rhandle = get_rust_handle(handle);
+pub fn load_css(css_styles: &mut CssStyles, aurl: &AllowedUrl) -> Result<(), LoadingError> {
+    io::acquire_data(aurl, None)
+        .and_then(|data| {
+            let BinaryData {
+                data: bytes,
+                content_type,
+            } = data;
 
-    let aurl = match AllowedUrl::from_href(href_str, rhandle.base_url.borrow().as_ref()) {
-        Ok(a) => a,
-        Err(_) => {
-            rsvg_log!("Could not load \"{}\" for CSS data", href_str);
-            // FIXME: report errors; this should be a fatal error
-            return;
-        }
-    };
-
-    if let Ok(data) = acquire_data(handle, &aurl) {
-        let BinaryData {
-            data: bytes,
-            content_type,
-        } = data;
-
-        if content_type.as_ref().map(String::as_ref) != Some("text/css") {
-            rsvg_log!("\"{}\" is not of type text/css; ignoring", href_str);
-            // FIXME: report errors
-            return;
-        }
-
-        if let Ok(utf8) = String::from_utf8(bytes) {
-            css::parse_into_css_styles(css_styles, handle, &utf8);
-        } else {
-            rsvg_log!(
-                "\"{}\" does not contain valid UTF-8 CSS data; ignoring",
-                href_str
-            );
-            // FIXME: report errors
-            return;
-        }
-    } else {
-        rsvg_log!("Could not load \"{}\" for CSS data", href_str);
-        // FIXME: report errors from not being to acquire data; this should be a fatal error
-    }
+            if content_type.as_ref().map(String::as_ref) == Some("text/css") {
+                Ok(bytes)
+            } else {
+                rsvg_log!("\"{}\" is not of type text/css; ignoring", aurl);
+                Err(LoadingError::BadCss)
+            }
+        })
+        .and_then(|bytes| {
+            String::from_utf8(bytes).map_err(|_| {
+                rsvg_log!(
+                    "\"{}\" does not contain valid UTF-8 CSS data; ignoring",
+                    aurl
+                );
+                LoadingError::BadCss
+            })
+        })
+        .and_then(|utf8| {
+            css::parse_into_css_styles(css_styles, Some(aurl.url().clone()), &utf8);
+            Ok(()) // FIXME: return CSS parsing errors
+        })
 }
 
 #[no_mangle]
@@ -728,7 +730,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_free(raw_handle: *mut Handle) {
     Box::from_raw(raw_handle);
 }
 
-pub fn get_rust_handle<'a>(handle: *const RsvgHandle) -> &'a mut Handle {
+fn get_rust_handle<'a>(handle: *const RsvgHandle) -> &'a mut Handle {
     unsafe { &mut *(rsvg_handle_get_rust(handle) as *mut Handle) }
 }
 
@@ -742,21 +744,7 @@ pub unsafe extern "C" fn rsvg_handle_rust_set_base_url(
     assert!(!uri.is_null());
     let uri: String = from_glib_none(uri);
 
-    let url = match Url::parse(&uri) {
-        Ok(u) => u,
-
-        Err(e) => {
-            rsvg_log!(
-                "not setting base_uri to \"{}\" since it is invalid: {}",
-                uri,
-                e
-            );
-            return;
-        }
-    };
-
-    rsvg_log!("setting base_uri to \"{}\"", url);
-    *handle.base_url.borrow_mut() = Some(url);
+    handle.set_base_url(&uri);
 }
 
 #[no_mangle]
@@ -767,11 +755,36 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_base_gfile(
 
     match *handle.base_url.borrow() {
         None => ptr::null_mut(),
-
         Some(ref url) => GFile::new_for_uri(url.as_str()).to_glib_full(),
     }
 }
 
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_set_base_gfile(
+    raw_handle: *const Handle,
+    raw_gfile: *mut gio_sys::GFile,
+) {
+    let handle = &*raw_handle;
+
+    assert!(!raw_gfile.is_null());
+
+    let file: GFile = from_glib_none(raw_gfile);
+
+    handle.set_base_gfile(&file);
+}
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_get_base_url(
+    raw_handle: *const Handle,
+) -> *const libc::c_char {
+    let handle = &*raw_handle;
+
+    match *handle.base_url_cstring.borrow() {
+        None => ptr::null(),
+        Some(ref url) => url.as_ptr(),
+    }
+}
+
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_set_dpi_x(raw_handle: *const Handle, dpi_x: f64) {
     let handle = &*(raw_handle as *const Handle);
@@ -804,21 +817,64 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_dpi_y(raw_handle: *const Handle) -
 pub unsafe extern "C" fn rsvg_handle_rust_get_flags(raw_handle: *const Handle) -> u32 {
     let rhandle = &*raw_handle;
 
-    rhandle.load_options.get().to_flags()
+    rhandle.load_flags.get().to_flags()
 }
 
 #[no_mangle]
 pub unsafe extern "C" fn rsvg_handle_rust_set_flags(raw_handle: *const Handle, flags: u32) {
     let rhandle = &*raw_handle;
 
-    rhandle.load_options.set(LoadOptions::from_flags(flags));
+    rhandle.load_flags.set(LoadFlags::from_flags(flags));
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_handle_rust_get_load_state(raw_handle: *const Handle) -> LoadState {
+pub unsafe extern "C" fn rsvg_handle_rust_set_size_callback(
+    raw_handle: *mut Handle,
+    size_func: RsvgSizeFunc,
+    user_data: glib_sys::gpointer,
+    destroy_notify: glib_sys::GDestroyNotify,
+) {
+    let rhandle = &mut *raw_handle;
+
+    *rhandle.size_callback.borrow_mut() = SizeCallback {
+        size_func,
+        user_data,
+        destroy_notify,
+    };
+}
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_set_testing(
+    raw_handle: *const Handle,
+    testing: glib_sys::gboolean,
+) {
     let rhandle = &*raw_handle;
 
-    rhandle.load_state.get()
+    rhandle.is_testing.set(from_glib(testing));
+}
+
+fn is_loaded(handle: &Handle) -> bool {
+    match handle.load_state.get() {
+        LoadState::Start => {
+            rsvg_g_warning("RsvgHandle has not been loaded");
+            false
+        }
+
+        LoadState::Loading => {
+            rsvg_g_warning("RsvgHandle is still loading; call rsvg_handle_close() first");
+            false
+        }
+
+        LoadState::ClosedOk => true,
+
+        LoadState::ClosedError => {
+            rsvg_g_warning(
+                "RsvgHandle could not read or parse the SVG; did you check for errors during the \
+                 loading stage?",
+            );
+            false
+        }
+    }
 }
 
 #[no_mangle]
@@ -896,6 +952,10 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_geometry_sub(
 ) -> glib_sys::gboolean {
     let rhandle = get_rust_handle(handle);
 
+    if !is_loaded(rhandle) {
+        return false.to_glib();
+    }
+
     let id: Option<String> = from_glib_none(id);
 
     match rhandle.get_geometry_sub(handle, id.as_ref().map(String::as_str)) {
@@ -931,15 +991,18 @@ pub unsafe extern "C" fn rsvg_handle_rust_has_sub(
     handle: *mut RsvgHandle,
     id: *const libc::c_char,
 ) -> glib_sys::gboolean {
-    if id.is_null() {
-        false.to_glib()
-    } else {
-        let id: String = from_glib_none(id);
+    let rhandle = get_rust_handle(handle);
 
-        let rhandle = get_rust_handle(handle);
-
-        rhandle.has_sub(handle, &id).to_glib()
+    if !is_loaded(rhandle) {
+        return false.to_glib();
     }
+
+    if id.is_null() {
+        return false.to_glib();
+    }
+
+    let id: String = from_glib_none(id);
+    rhandle.has_sub(&id).to_glib()
 }
 
 #[no_mangle]
@@ -952,6 +1015,10 @@ pub unsafe extern "C" fn rsvg_handle_rust_render_cairo_sub(
     let cr = from_glib_none(cr);
     let id: Option<String> = from_glib_none(id);
 
+    if !is_loaded(rhandle) {
+        return false.to_glib();
+    }
+
     match rhandle.render_cairo_sub(handle, &cr, id.as_ref().map(String::as_str)) {
         Ok(()) => true.to_glib(),
 
@@ -968,11 +1035,124 @@ pub unsafe extern "C" fn rsvg_handle_rust_get_pixbuf_sub(
     id: *const libc::c_char,
 ) -> *mut gdk_pixbuf_sys::GdkPixbuf {
     let rhandle = get_rust_handle(handle);
-
     let id: Option<String> = from_glib_none(id);
 
+    if !is_loaded(rhandle) {
+        return ptr::null_mut();
+    }
+
     match rhandle.get_pixbuf_sub(handle, id.as_ref().map(String::as_str)) {
         Ok(pixbuf) => pixbuf.to_glib_full(),
         Err(_) => ptr::null_mut(),
     }
 }
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_get_dimensions(
+    handle: *mut RsvgHandle,
+    dimension_data: *mut RsvgDimensionData,
+) {
+    let rhandle = get_rust_handle(handle);
+
+    if !is_loaded(rhandle) {
+        return;
+    }
+
+    // This function is probably called from the cairo_render functions.
+    // To prevent an infinite loop we are saving the state.
+    if rhandle.in_loop.get() {
+        // Called within the size function, so return a standard size
+        (*dimension_data).width = 1;
+        (*dimension_data).height = 1;
+        (*dimension_data).em = 1.0;
+        (*dimension_data).ex = 1.0;
+        return;
+    }
+
+    rhandle.in_loop.set(true);
+    rsvg_handle_rust_get_dimensions_sub(handle, dimension_data, ptr::null());
+    rhandle.in_loop.set(false);
+}
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_get_dimensions_sub(
+    handle: *mut RsvgHandle,
+    dimension_data: *mut RsvgDimensionData,
+    id: *const libc::c_char,
+) -> glib_sys::gboolean {
+    let rhandle = get_rust_handle(handle);
+
+    if !is_loaded(rhandle) {
+        return false.to_glib();
+    }
+
+    let mut ink_r = RsvgRectangle {
+        x: 0.0,
+        y: 0.0,
+        width: 0.0,
+        height: 0.0,
+    };
+
+    let res = rsvg_handle_rust_get_geometry_sub(handle, &mut ink_r, ptr::null_mut(), id);
+    if from_glib(res) {
+        let (w, h) = rhandle
+            .size_callback
+            .borrow()
+            .call(ink_r.width as libc::c_int, ink_r.height as libc::c_int);
+
+        (*dimension_data).width = w;
+        (*dimension_data).height = h;
+        (*dimension_data).em = ink_r.width;
+        (*dimension_data).ex = ink_r.height;
+    } else {
+        (*dimension_data).width = 0;
+        (*dimension_data).height = 0;
+        (*dimension_data).em = 0.0;
+        (*dimension_data).ex = 0.0;
+    }
+
+    res
+}
+
+#[no_mangle]
+pub unsafe extern "C" fn rsvg_handle_rust_get_position_sub(
+    handle: *mut RsvgHandle,
+    position: *mut RsvgPositionData,
+    id: *const libc::c_char,
+) -> glib_sys::gboolean {
+    let rhandle = get_rust_handle(handle);
+
+    if !is_loaded(rhandle) {
+        return false.to_glib();
+    }
+
+    // Short-cut when no id is given
+    if id.is_null() || *id == 0 {
+        (*position).x = 0;
+        (*position).y = 0;
+        return true.to_glib();
+    }
+
+    let mut ink_r = RsvgRectangle {
+        x: 0.0,
+        y: 0.0,
+        width: 0.0,
+        height: 0.0,
+    };
+
+    let res = rsvg_handle_rust_get_geometry_sub(handle, &mut ink_r, ptr::null_mut(), id);
+    if from_glib(res) {
+        (*position).x = ink_r.x as libc::c_int;
+        (*position).y = ink_r.y as libc::c_int;
+
+        let width = ink_r.width as libc::c_int;
+        let height = ink_r.height as libc::c_int;
+
+        rhandle.size_callback.borrow().call(width, height);
+    } else {
+        (*position).x = 0;
+        (*position).y = 0;
+    }
+
+    res
+}
