@@ -1,5 +1,4 @@
 use data_url;
-use libc;
 
 use gio::{
     self,
@@ -14,7 +13,7 @@ use gio::{
     ZlibCompressorFormat,
     ZlibDecompressor,
 };
-use glib::{self, translate::*, Bytes as GBytes, Cast};
+use glib::{self, Bytes as GBytes, Cast};
 
 use allowed_url::AllowedUrl;
 use error::{LoadingError, RsvgError};
@@ -44,22 +43,6 @@ fn decode_data_uri(uri: &str) -> Result<BinaryData, LoadingError> {
     })
 }
 
-pub fn binary_data_to_glib(
-    binary_data: &BinaryData,
-    out_mime_type: *mut *mut libc::c_char,
-    out_size: *mut usize,
-) -> *mut libc::c_char {
-    unsafe {
-        if !out_mime_type.is_null() {
-            *out_mime_type = binary_data.content_type.to_glib_full();
-        }
-
-        *out_size = binary_data.data.len();
-
-        ToGlibContainerFromSlice::to_glib_full_from_slice(&binary_data.data) as *mut libc::c_char
-    }
-}
-
 // Header of a gzip data stream
 const GZ_MAGIC_0: u8 = 0x1f;
 const GZ_MAGIC_1: u8 = 0x8b;
