@@ -36,24 +36,32 @@ extern crate lazy_static;
 
 pub use color::{rsvg_css_parse_color, ColorKind, ColorSpec};
 
+pub use dpi::rsvg_rust_set_default_dpi_x_y;
+
 pub use handle::{
     rsvg_handle_rust_close,
     rsvg_handle_rust_free,
     rsvg_handle_rust_get_base_gfile,
+    rsvg_handle_rust_get_base_url,
+    rsvg_handle_rust_get_dimensions,
+    rsvg_handle_rust_get_dimensions_sub,
     rsvg_handle_rust_get_dpi_x,
     rsvg_handle_rust_get_dpi_y,
     rsvg_handle_rust_get_flags,
     rsvg_handle_rust_get_geometry_sub,
-    rsvg_handle_rust_get_load_state,
     rsvg_handle_rust_get_pixbuf_sub,
+    rsvg_handle_rust_get_position_sub,
     rsvg_handle_rust_has_sub,
     rsvg_handle_rust_new,
     rsvg_handle_rust_read_stream_sync,
     rsvg_handle_rust_render_cairo_sub,
+    rsvg_handle_rust_set_base_gfile,
     rsvg_handle_rust_set_base_url,
     rsvg_handle_rust_set_dpi_x,
     rsvg_handle_rust_set_dpi_y,
     rsvg_handle_rust_set_flags,
+    rsvg_handle_rust_set_size_callback,
+    rsvg_handle_rust_set_testing,
     rsvg_handle_rust_write,
 };
 
@@ -119,7 +127,7 @@ mod text;
 mod transform;
 mod tree;
 pub mod tree_utils;
-mod unitinterval;
+mod unit_interval;
 mod util;
 mod viewbox;
 mod viewport;
