@@ -1,6 +1,6 @@
 use cssparser::{BasicParseError, Parser, ParserInput, Token};
 
-use std::str::{self, FromStr};
+use std::str;
 
 use attributes::Attribute;
 use error::{NodeError, ValueErrorKind};
@@ -41,16 +41,21 @@ pub trait Parse: Sized {
     }
 }
 
-/// Extend `cssparser::Parser` with a `expect_finite_number` method,
-/// to avoid infinities.
 pub trait CssParserExt {
+    /// Avoid infinities.
     fn expect_finite_number(&mut self) -> Result<f32, ValueErrorKind>;
+
+    fn optional_comma(&mut self);
 }
 
 impl<'i, 't> CssParserExt for Parser<'i, 't> {
     fn expect_finite_number(&mut self) -> Result<f32, ValueErrorKind> {
         finite_f32(self.expect_number()?)
     }
+
+    fn optional_comma(&mut self) {
+        let _ = self.try(|p| p.expect_comma());
+    }
 }
 
 pub fn finite_f32(n: f32) -> Result<f32, ValueErrorKind> {
@@ -61,6 +66,54 @@ pub fn finite_f32(n: f32) -> Result<f32, ValueErrorKind> {
     }
 }
 
+pub trait ParseValue<T: Parse<Err = ValueErrorKind>> {
+    /// Parses a `value` string into a type `T`.
+    ///
+    /// Some value types need some extra `data` to be parsed.  This
+    /// corresponds to the `<T as Parse>::Data` associated type.  For
+    /// example, an `Length` has an associated `type Data =
+    /// LengthDir`, so to parse a length value, you could specify
+    /// `LengthDir::Horizontal` for `data`, for example.
+    fn parse(&self, value: &str, data: <T as Parse>::Data) -> Result<T, NodeError>;
+
+    /// Parses a `value` string into a type `T` with an optional validation function.
+    ///
+    /// Some value types need some extra `data` to be parsed.  This
+    /// corresponds to the `<T as Parse>::Data` associated type.  For
+    /// example, an `Length` has an associated `type Data =
+    /// LengthDir`, so to parse a length value, you could specify
+    /// `LengthDir::Horizontal` for `data`, for example.
+    fn parse_and_validate<F: FnOnce(T) -> Result<T, ValueErrorKind>>(
+        &self,
+        value: &str,
+        data: <T as Parse>::Data,
+        validate: F,
+    ) -> Result<T, NodeError>;
+}
+
+impl<T: Parse<Err = ValueErrorKind>> ParseValue<T> for Attribute {
+    fn parse(&self, value: &str, data: <T as Parse>::Data) -> Result<T, NodeError> {
+        let mut input = ParserInput::new(value);
+        let mut parser = Parser::new(&mut input);
+
+        T::parse(&mut parser, data).map_err(|e| NodeError::attribute_error(*self, e))
+    }
+
+    fn parse_and_validate<F: FnOnce(T) -> Result<T, ValueErrorKind>>(
+        &self,
+        value: &str,
+        data: <T as Parse>::Data,
+        validate: F,
+    ) -> Result<T, NodeError> {
+        let mut input = ParserInput::new(value);
+        let mut parser = Parser::new(&mut input);
+
+        T::parse(&mut parser, data)
+            .and_then(validate)
+            .map_err(|e| NodeError::attribute_error(*self, e))
+    }
+}
+
 impl Parse for f64 {
     type Data = ();
     type Err = ValueErrorKind;
@@ -72,53 +125,6 @@ impl Parse for f64 {
     }
 }
 
-/// Parses a `value` string into a type `T`.
-///
-/// Some value types need some extra `data` to be parsed.  This
-/// corresponds to the `<T as Parse>::Data` associated type.  For
-/// example, an `Length` has an associated `type Data =
-/// LengthDir`, so to parse a length value, you could specify
-/// `LengthDir::Horizontal` for `data`, for example.
-pub fn parse<T>(key: &str, value: &str, data: <T as Parse>::Data) -> Result<T, NodeError>
-where
-    T: Parse<Err = ValueErrorKind>,
-{
-    let mut input = ParserInput::new(value);
-    let mut parser = Parser::new(&mut input);
-
-    T::parse(&mut parser, data)
-        .map_err(|e| NodeError::attribute_error(Attribute::from_str(key).unwrap(), e))
-}
-
-/// Parses a `value` string into a type `T` with an optional validation function.
-///
-/// Some value types need some extra `data` to be parsed.  This
-/// corresponds to the `<T as Parse>::Data` associated type.  For
-/// example, an `Length` has an associated `type Data =
-/// LengthDir`, so to parse a length value, you could specify
-/// `LengthDir::Horizontal` for `data`, for example.
-pub fn parse_and_validate<T, F>(
-    key: &str,
-    value: &str,
-    data: <T as Parse>::Data,
-    validate: F,
-) -> Result<T, NodeError>
-where
-    T: Parse<Err = ValueErrorKind>,
-    F: FnOnce(T) -> Result<T, ValueErrorKind>,
-{
-    let mut input = ParserInput::new(value);
-    let mut parser = Parser::new(&mut input);
-
-    T::parse(&mut parser, data)
-        .and_then(validate)
-        .map_err(|e| NodeError::attribute_error(Attribute::from_str(key).unwrap(), e))
-}
-
-pub fn optional_comma(parser: &mut Parser<'_, '_>) {
-    let _ = parser.try(|p| p.expect_comma());
-}
-
 // number
 //
 // https://www.w3.org/TR/SVG11/types.html#DataTypeNumber
@@ -194,37 +200,6 @@ pub fn integer_optional_integer(s: &str) -> Result<(i32, i32), ValueErrorKind> {
     }
 }
 
-// Parse a list-of-points as for polyline and polygon elements
-// https://www.w3.org/TR/SVG/shapes.html#PointsBNF
-
-pub fn list_of_points(string: &str) -> Result<Vec<(f64, f64)>, ValueErrorKind> {
-    let mut input = ParserInput::new(string);
-    let mut parser = Parser::new(&mut input);
-
-    let mut v = Vec::new();
-
-    loop {
-        let x = f64::from(parser.expect_finite_number()?);
-
-        optional_comma(&mut parser);
-
-        let y = f64::from(parser.expect_finite_number()?);
-
-        v.push((x, y));
-
-        if parser.is_exhausted() {
-            break;
-        }
-
-        match parser.next_including_whitespace() {
-            Ok(&Token::WhiteSpace(_)) => (),
-            _ => optional_comma(&mut parser),
-        }
-    }
-
-    Ok(v)
-}
-
 // Lists of number values
 
 #[derive(Eq, PartialEq)]
@@ -263,7 +238,7 @@ pub fn number_list(
 
     for i in 0.. {
         if i != 0 {
-            optional_comma(parser);
+            parser.optional_comma();
         }
 
         v.push(f64::from(parser.expect_finite_number().map_err(|_| {
@@ -328,28 +303,6 @@ mod tests {
         assert!(number_optional_number("1 2 x").is_err());
     }
 
-    #[test]
-    fn parses_list_of_points() {
-        assert_eq!(list_of_points(" 1 2 "), Ok(vec![(1.0, 2.0)]));
-        assert_eq!(list_of_points("1 2 3 4"), Ok(vec![(1.0, 2.0), (3.0, 4.0)]));
-        assert_eq!(list_of_points("1,2,3,4"), Ok(vec![(1.0, 2.0), (3.0, 4.0)]));
-        assert_eq!(list_of_points("1,2 3,4"), Ok(vec![(1.0, 2.0), (3.0, 4.0)]));
-        assert_eq!(
-            list_of_points("1,2 -3,4"),
-            Ok(vec![(1.0, 2.0), (-3.0, 4.0)])
-        );
-        assert_eq!(
-            list_of_points("1,2,-3,4"),
-            Ok(vec![(1.0, 2.0), (-3.0, 4.0)])
-        );
-    }
-
-    #[test]
-    fn errors_on_invalid_list_of_points() {
-        assert!(list_of_points("-1-2-3-4").is_err());
-        assert!(list_of_points("1 2-3,-4").is_err());
-    }
-
     #[test]
     fn parses_number_list() {
         assert_eq!(
