@@ -13,14 +13,13 @@ use defs::Fragment;
 use drawing_ctx::{DrawingCtx, NodeStack};
 use error::{AttributeResultExt, RenderingError};
 use float_eq_cairo::ApproxEqCairo;
-use handle::RsvgHandle;
 use length::*;
 use node::*;
 use paint_server::PaintSource;
-use parsers::{parse, parse_and_validate};
+use parsers::ParseValue;
 use property_bag::PropertyBag;
 use state::ComputedValues;
-use unitinterval::UnitInterval;
+use unit_interval::UnitInterval;
 use viewbox::*;
 
 coord_units!(PatternUnits, CoordUnits::ObjectBoundingBox);
@@ -173,12 +172,7 @@ impl NodePattern {
 }
 
 impl NodeTrait for NodePattern {
-    fn set_atts(
-        &self,
-        node: &RsvgNode,
-        _: *const RsvgHandle,
-        pbag: &PropertyBag<'_>,
-    ) -> NodeResult {
+    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
         // pattern element has overflow:hidden
         // https://www.w3.org/TR/SVG/styling.html#UAStyleSheet
         node.set_overflow_hidden();
@@ -189,33 +183,28 @@ impl NodeTrait for NodePattern {
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::PatternUnits => p.units = Some(parse("patternUnits", value, ())?),
+                Attribute::PatternUnits => p.units = Some(attr.parse(value, ())?),
 
-                Attribute::PatternContentUnits => {
-                    p.content_units = Some(parse("patternContentUnits", value, ())?)
-                }
+                Attribute::PatternContentUnits => p.content_units = Some(attr.parse(value, ())?),
 
-                Attribute::ViewBox => p.vbox = Some(Some(parse("viewBox", value, ())?)),
+                Attribute::ViewBox => p.vbox = Some(Some(attr.parse(value, ())?)),
 
                 Attribute::PreserveAspectRatio => {
-                    p.preserve_aspect_ratio = Some(parse("preserveAspectRatio", value, ())?)
+                    p.preserve_aspect_ratio = Some(attr.parse(value, ())?)
                 }
 
-                Attribute::PatternTransform => {
-                    p.affine = Some(parse("patternTransform", value, ())?)
-                }
+                Attribute::PatternTransform => p.affine = Some(attr.parse(value, ())?),
 
                 Attribute::XlinkHref => {
                     p.fallback = Some(Fragment::parse(value).attribute(Attribute::XlinkHref)?);
                 }
 
-                Attribute::X => p.x = Some(parse("x", value, LengthDir::Horizontal)?),
+                Attribute::X => p.x = Some(attr.parse(value, LengthDir::Horizontal)?),
 
-                Attribute::Y => p.y = Some(parse("y", value, LengthDir::Vertical)?),
+                Attribute::Y => p.y = Some(attr.parse(value, LengthDir::Vertical)?),
 
                 Attribute::Width => {
-                    p.width = Some(parse_and_validate(
-                        "width",
+                    p.width = Some(attr.parse_and_validate(
                         value,
                         LengthDir::Horizontal,
                         Length::check_nonnegative,
@@ -223,8 +212,7 @@ impl NodeTrait for NodePattern {
                 }
 
                 Attribute::Height => {
-                    p.height = Some(parse_and_validate(
-                        "height",
+                    p.height = Some(attr.parse_and_validate(
                         value,
                         LengthDir::Vertical,
                         Length::check_nonnegative,
@@ -239,13 +227,15 @@ impl NodeTrait for NodePattern {
     }
 }
 
-impl PaintSource<Pattern> for NodePattern {
+impl PaintSource for NodePattern {
+    type Source = Pattern;
+
     fn resolve(
         &self,
         node: &RsvgNode,
         draw_ctx: &mut DrawingCtx,
         _bbox: &BoundingBox,
-    ) -> Option<Pattern> {
+    ) -> Result<Option<Self::Source>, RenderingError> {
         let node_pattern = node.get_impl::<NodePattern>().unwrap();
         let pattern = &*node_pattern.pattern.borrow();
         let mut result = pattern.clone();
@@ -255,32 +245,32 @@ impl PaintSource<Pattern> for NodePattern {
             if let Some(acquired) =
                 draw_ctx.get_acquired_node_of_type(result.fallback.as_ref(), NodeType::Pattern)
             {
-                let node = acquired.get();
+                let a_node = acquired.get();
 
-                if stack.contains(node) {
-                    // FIXME: return a Result here with RenderingError::CircularReference
-                    // FIXME: print the pattern's name
-                    rsvg_log!("circular reference in pattern");
-                    result.resolve_from_defaults();
-                    break;
+                if stack.contains(a_node) {
+                    rsvg_log!(
+                        "circular reference in pattern {}",
+                        node.get_human_readable_name()
+                    );
+                    return Err(RenderingError::CircularReference);
                 }
 
-                node.with_impl(|i: &NodePattern| {
+                a_node.with_impl(|i: &NodePattern| {
                     result.resolve_from_fallback(&*i.pattern.borrow())
                 });
 
-                stack.push(node);
+                stack.push(a_node);
             } else {
                 result.resolve_from_defaults();
             }
         }
 
-        Some(result)
+        Ok(Some(result))
     }
 
     fn set_pattern_on_draw_context(
         &self,
-        pattern: &Pattern,
+        pattern: &Self::Source,
         values: &ComputedValues,
         draw_ctx: &mut DrawingCtx,
         _opacity: &UnitInterval,
