@@ -3,16 +3,7 @@ use libc;
 use std::ffi::CStr;
 use std::str;
 
-// In paint servers (patterns, gradients, etc.), we have an
-// Option<String> for fallback names.  This is a utility function to
-// clone one of those.
-pub fn clone_fallback_name(fallback: &Option<String>) -> Option<String> {
-    if let Some(ref fallback_name) = *fallback {
-        Some(fallback_name.clone())
-    } else {
-        None
-    }
-}
+use glib::translate::*;
 
 /// Converts a `char *` which is known to be valid UTF-8 into a `&str`
 ///
@@ -35,3 +26,13 @@ pub fn clamp<T: PartialOrd>(val: T, low: T, high: T) -> T {
         val
     }
 }
+
+extern "C" {
+    fn rsvg_g_warning_from_c(msg: *const libc::c_char);
+}
+
+pub fn rsvg_g_warning(msg: &str) {
+    unsafe {
+        rsvg_g_warning_from_c(msg.to_glib_none().0);
+    }
+}
